unit LEDSSD;

interface

uses
  Windows, Graphics, Classes, Controls;

type
  TLEDSSD = class(TGraphicControl)
  private
    Value: Byte;
    CASegment: TColor;
    CISegment: TColor;
    CBackground: TColor;
    SegWidth: Byte;
    Padding: Byte;
    Spacing: Byte;
    SegmentPoints: array[0..6,0..5] of TPoint;
    procedure SetValue(const SDigit: Byte);
    procedure SetBackgroundColor(const Value: TColor);
    procedure SetActiveColor(const Value: TColor);
    procedure SetInactiveColor(const Value: TColor);
    procedure SetSegWidth(const SWidth: Byte);
    procedure SetSpacing(const SWidth: Byte);
    procedure SetPadding(const PWidth: Byte);
    procedure Geometry;
  protected
    procedure Paint; override;
    procedure Resize; override;
  public
    constructor Create(COwner: TComponent); override;
    destructor Destroy; override;
  published
    property Digit: Byte read Value write SetValue;
    property ColorBackground: TColor read CBackground write SetBackgroundColor;
    property ColorActiveSegment: TColor read CASegment write SetActiveColor;
    property ColorInactiveSegment: TColor read CISegment write SetInactiveColor;
    property WidthSegment: Byte read SegWidth write SetSegWidth;
    property WidthPadding: Byte read Padding write SetPadding;
    property WidthSpacing: Byte read Spacing write SetSpacing;
    property Cursor;
    property Enabled;
    property Visible;
end;

const
  DigitAsSegment: array[0..15, 0..6] of boolean = (
    (true,  true,  true,  true,  true,  true,  false),
    (false, true,  true,  false, false, false, false),
    (true,  true,  false, true,  true,  false, true ),
    (true,  true,  true,  true,  false, false, true ),
    (false, true,  true,  false, false, true,  true ),
    (true,  false, true,  true,  false, true,  true ),
    (true,  false, true,  true,  true,  true,  true ),
    (true,  true,  true,  false, false, false, false),
    (true,  true,  true,  true,  true,  true,  true ),
    (true,  true,  true,  true,  false, true,  true ),
    (true,  true,  true,  false,  true,  true,  true ),
    (false,  false,  true,  true,  true,  true,  true ),
    (true,  false,  false,  true,  true,  true,  false ),
    (false,  true,  true,  true,  true,  false,  true ),
    (true,  false,  false,  true,  true,  true,  true ),
    (true,  false,  false,  false,  true,  true,  true )
    );

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('PLS', [TLEDSSD]);
end;

//----------------------Konstruktor, destruktor -----------------

constructor TLEDSSD.Create(COwner: TComponent);
begin
  inherited Create(COwner);
  CASegment := clLime;
  CISegment := $1E361E;
  CBackground := clBlack;
  SegWidth := 10;
  Padding := 5;
  Spacing := 2;
  Width := 100;
  Height := 140;
end;

destructor TLEDSSD.Destroy;
begin
  inherited Destroy;
end;

// ----------------- Setterek -------------------------------

procedure TLEDSSD.SetValue(const SDigit: Byte);
begin
  if (Value <> SDigit) and (SDigit <= 15) then
  begin
    Value := SDigit;
    Geometry;
    Invalidate;
  end;
end;

procedure TLEDSSD.SetBackgroundColor(const Value: TColor);
begin
  if CBackground <> Value then
  begin
    CBackground := Value;
    Invalidate;
  end;
end;

procedure TLEDSSD.SetActiveColor(const Value: TColor);
begin
  if CASegment <> Value then
  begin
    CASegment := Value;
    Invalidate;
  end;
end;

procedure TLEDSSD.SetInactiveColor(const Value: TColor);
begin
  if CISegment <> Value then
  begin
    CISegment := Value;
    Invalidate;
  end;
end;

procedure TLEDSSD.SetSegWidth(const SWidth: Byte);
begin
  if SegWidth <> SWidth then
  begin
    SegWidth := SWidth;
    Geometry;
    Invalidate;
  end;
end;

procedure TLEDSSD.SetSpacing(const SWidth: Byte);
begin
  if Spacing <> SWidth then
  begin
    Spacing := SWidth;
    Geometry;
    Invalidate;
  end;
end;

procedure TLEDSSD.SetPadding(const PWidth: Byte);
begin
  if Padding <> PWidth then
  begin
    Padding := PWidth;
    Geometry;
    Invalidate;
  end;
end;

// -------------------- Felldefinilt metdusok -------------

procedure TLEDSSD.Paint();
var
  i: Byte;
begin
  inherited;
  with Canvas do
  begin
    Brush.Color := CBackGround;
    Pen.Color := CBackground;
    Rectangle(0, 0, Width, Height);
    if Visible then
    begin
    for i := 0 to 6 do
    begin
      if Enabled then
      begin
        if DigitAsSegment[Digit, i] then
          Brush.Color := CASegment
        else
          Brush.Color := CISegment;
      end
      else
        Brush.Color := CISegment;

      Polygon(SegmentPoints[i]);
    end;
  end;
  end;
end;

procedure TLEDSSD.Resize;
begin
  inherited;
  Geometry;
  Invalidate;
end;

// --------------------------- Geometria ----------------------
procedure TLEDSSD.Geometry;
var
  X: Integer;
  Y: Integer;
  SLength: Integer;
  SHeight: Integer;
begin
  SLength := Width - Padding*2 - Spacing * 2 - SegWidth;
  SHeight := ((Height - Padding*2) div 2)  - Spacing * 2 - SegWidth;
  //Szegmens a
  X := Padding + (SegWidth div 2) + Spacing;
  Y := Padding + (SegWidth div 2);
  SegmentPoints[0,0] := Point(X,Y);
  SegmentPoints[0,1] := Point(X + (SegWidth div 2),Y - (SegWidth div 2));
  SegmentPoints[0,2] := Point(X + SLength - (SegWidth div 2),Y - (SegWidth div 2));
  SegmentPoints[0,3] := Point(X + SLength, Y);
  SegmentPoints[0,4] := Point(X + SLength - (SegWidth div 2),Y + (SegWidth div 2));
  SegmentPoints[0,5] := Point(X + (SegWidth div 2),Y + (SegWidth div 2));
  //Szegmens b
  X := Width - (Padding + (SegWidth div 2));
  Y := Padding + (SegWidth div 2) + Spacing;
  SegmentPoints[1,0] := Point(X,Y);
  SegmentPoints[1,1] := Point(X + (SegWidth div 2),Y + (SegWidth div 2));
  SegmentPoints[1,2] := Point(X + (SegWidth div 2),Y + SHeight);
  SegmentPoints[1,3] := Point(X, Y + SHeight + (SegWidth div 2));
  SegmentPoints[1,4] := Point(X - (SegWidth div 2),Y + SHeight);
  SegmentPoints[1,5] := Point(X - (SegWidth div 2),Y + (SegWidth div 2));
  //Szegmens c
  X := Width - (Padding + (SegWidth div 2));
  Y := (Height div 2) + Spacing;
  SegmentPoints[2,0] := Point(X,Y);
  SegmentPoints[2,1] := Point(X + (SegWidth div 2),Y + (SegWidth div 2));
  SegmentPoints[2,2] := Point(X + (SegWidth div 2),Y + SHeight);
  SegmentPoints[2,3] := Point(X, Y + SHeight + (SegWidth div 2));
  SegmentPoints[2,4] := Point(X - (SegWidth div 2),Y + SHeight);
  SegmentPoints[2,5] := Point(X - (SegWidth div 2),Y + (SegWidth div 2));
  //Szegmens d
  X := Padding + (SegWidth div 2) + Spacing;
  Y := Height- (Padding + (SegWidth div 2));
  SegmentPoints[3,0] := Point(X,Y);
  SegmentPoints[3,1] := Point(X + (SegWidth div 2),Y - (SegWidth div 2));
  SegmentPoints[3,2] := Point(X + SLength - (SegWidth div 2),Y - (SegWidth div 2));
  SegmentPoints[3,3] := Point(X + SLength, Y);
  SegmentPoints[3,4] := Point(X + SLength - (SegWidth div 2),Y + (SegWidth div 2));
  SegmentPoints[3,5] := Point(X + (SegWidth div 2),Y + (SegWidth div 2));
  //Szegmens e
  X := Padding + (SegWidth div 2);
  Y := (Height div 2) + Spacing;
  SegmentPoints[4,0] := Point(X,Y);
  SegmentPoints[4,1] := Point(X + (SegWidth div 2),Y + (SegWidth div 2));
  SegmentPoints[4,2] := Point(X + (SegWidth div 2),Y + SHeight);
  SegmentPoints[4,3] := Point(X, Y + SHeight + (SegWidth div 2));
  SegmentPoints[4,4] := Point(X - (SegWidth div 2),Y + SHeight);
  SegmentPoints[4,5] := Point(X - (SegWidth div 2),Y + (SegWidth div 2));
  //Szegmens f
  X := Padding + (SegWidth div 2);
  Y := Padding + (SegWidth div 2) + Spacing;
  SegmentPoints[5,0] := Point(X,Y);
  SegmentPoints[5,1] := Point(X + (SegWidth div 2),Y + (SegWidth div 2));
  SegmentPoints[5,2] := Point(X + (SegWidth div 2),Y + SHeight);
  SegmentPoints[5,3] := Point(X, Y + SHeight + (SegWidth div 2));
  SegmentPoints[5,4] := Point(X - (SegWidth div 2),Y + SHeight);
  SegmentPoints[5,5] := Point(X - (SegWidth div 2),Y + (SegWidth div 2));
  //Szegmens g
  X := Padding + (SegWidth div 2) + Spacing;
  Y := (Height div 2);
  SegmentPoints[6,0] := Point(X,Y);
  SegmentPoints[6,1] := Point(X + (SegWidth div 2),Y - (SegWidth div 2));
  SegmentPoints[6,2] := Point(Width - X - (SegWidth div 2),Y - (SegWidth div 2));
  SegmentPoints[6,3] := Point(Width - X, Y);
  SegmentPoints[6,4] := Point(Width - X - (SegWidth div 2),Y + (SegWidth div 2));
  SegmentPoints[6,5] := Point(X + (SegWidth div 2),Y + (SegWidth div 2));
end;

end.
